@@ -72,16 +72,6 @@ enum {
 	LAST_PROP
 };
 
-#define DBUS_PROP_FLAGS "Flags"
-#define DBUS_PROP_WPA_FLAGS "WpaFlags"
-#define DBUS_PROP_RSN_FLAGS "RsnFlags"
-#define DBUS_PROP_SSID "Ssid"
-#define DBUS_PROP_FREQUENCY "Frequency"
-#define DBUS_PROP_HW_ADDRESS "HwAddress"
-#define DBUS_PROP_MODE "Mode"
-#define DBUS_PROP_MAX_BITRATE "MaxBitrate"
-#define DBUS_PROP_STRENGTH "Strength"
-
 /**
  * nm_access_point_new:
  * @connection: the #DBusGConnection
@@ -246,9 +236,9 @@ nm_access_point_get_mode (NMAccessPoint *ap)
  * nm_access_point_get_max_bitrate:
  * @ap: a #NMAccessPoint
  *
- * Gets the maximum bit rate of the access point.
+ * Gets the maximum bit rate of the access point in kbit/s.
  *
- * Returns: the maximum bit rate
+ * Returns: the maximum bit rate (kbit/s)
  **/
 guint32
 nm_access_point_get_max_bitrate (NMAccessPoint *ap)
@@ -539,12 +529,7 @@ constructed (GObject *object)
 	G_OBJECT_CLASS (nm_access_point_parent_class)->constructed (object);
 
 	priv = NM_ACCESS_POINT_GET_PRIVATE (object);
-
-	priv->proxy = dbus_g_proxy_new_for_name (nm_object_get_connection (NM_OBJECT (object)),
-									    NM_DBUS_SERVICE,
-									    nm_object_get_path (NM_OBJECT (object)),
-									    NM_DBUS_INTERFACE_ACCESS_POINT);
-
+	priv->proxy = _nm_object_new_proxy (NM_OBJECT (object), NULL, NM_DBUS_INTERFACE_ACCESS_POINT);
 	register_properties (NM_ACCESS_POINT (object));
 }
 
@@ -675,7 +660,7 @@ nm_access_point_class_init (NMAccessPointClass *ap_class)
 	/**
 	 * NMAccessPoint:max-bitrate:
 	 *
-	 * The maximum bit rate of the access point.
+	 * The maximum bit rate of the access point in kbit/s.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_MAX_BITRATE,
