@@ -55,6 +55,26 @@ typedef enum {
 	NM_SECRET_AGENT_ERROR_NO_SECRETS,         /*< nick=NoSecrets >*/
 } NMSecretAgentError;
 
+/**
+ * NMSecretAgentCapabilities:
+ * @NM_SECRET_AGENT_CAPABILITY_NONE: the agent supports no special capabilities
+ * @NM_SECRET_AGENT_CAPABILITY_VPN_HINTS: the agent supports sending hints given
+ * by the NMSecretAgentClass::get_secrets() class method to VPN plugin
+ * authentication dialogs.
+ * @NM_SECRET_AGENT_CAPABILITY_LAST: bounds checking value; should not be used.
+ *
+ * #NMSecretAgentCapabilities indicate various capabilities of the agent.
+ *
+ * Since: 0.9.10
+ */
+typedef enum /*< flags >*/ {
+	NM_SECRET_AGENT_CAPABILITY_NONE = 0x0,
+	NM_SECRET_AGENT_CAPABILITY_VPN_HINTS = 0x1,
+
+	/* boundary value */
+	NM_SECRET_AGENT_CAPABILITY_LAST = NM_SECRET_AGENT_CAPABILITY_VPN_HINTS
+} NMSecretAgentCapabilities;
+
 /**
  * NMSecretAgentGetSecretsFlags:
  * @NM_SECRET_AGENT_GET_SECRETS_FLAG_NONE: no special behavior; by default no
@@ -91,6 +111,7 @@ typedef enum /*< flags >*/ {
 #define NM_SECRET_AGENT_IDENTIFIER          "identifier"
 #define NM_SECRET_AGENT_AUTO_REGISTER       "auto-register"
 #define NM_SECRET_AGENT_REGISTERED          "registered"
+#define NM_SECRET_AGENT_CAPABILITIES        "capabilities"
 
 #define NM_SECRET_AGENT_REGISTRATION_RESULT "registration-result"
 
@@ -101,7 +122,10 @@ typedef struct {
 /**
  * NMSecretAgentGetSecretsFunc:
  * @agent: the secret agent object
- * @connection: the connection for which secrets were requested
+ * @connection: (transfer none): the connection for which secrets were requested,
+ * note that this object will be unrefed after the callback has returned, use
+ * g_object_ref()/g_object_unref() if you want to use this object after the callback
+ * has returned
  * @secrets: (element-type utf8 GLib.HashTable): the #GHashTable containing
  * the requested secrets in the same format as an #NMConnection hash (as
  * created by nm_connection_to_hash() for example).  Each key in @secrets
@@ -109,7 +133,7 @@ typedef struct {
  * and each value should be a #GHashTable.  The sub-hashes map string:#GValue
  * where the string is the setting property name (like "psk") and the value
  * is the secret
- * @error: if the secrets request failed, give a descriptive error here 
+ * @error: if the secrets request failed, give a descriptive error here
  * @user_data: caller-specific data to be passed to the function
  *
  * Called as a result of a request by NM to retrieve secrets.  When the
@@ -150,8 +174,11 @@ typedef void (*NMSecretAgentGetSecretsFunc) (NMSecretAgent *agent,
 /**
  * NMSecretAgentSaveSecretsFunc:
  * @agent: the secret agent object
- * @connection: the connection for which secrets were to be saved
- * @error: if the saving secrets failed, give a descriptive error here 
+ * @connection: (transfer none): the connection for which secrets were to be saved,
+ * note that this object will be unrefed after the callback has returned, use
+ * g_object_ref()/g_object_unref() if you want to use this object after the callback
+ * has returned
+ * @error: if the saving secrets failed, give a descriptive error here
  * @user_data: caller-specific data to be passed to the function
  *
  * Called as a result of a request by NM to save secrets.  When the
@@ -166,8 +193,11 @@ typedef void (*NMSecretAgentSaveSecretsFunc) (NMSecretAgent *agent,
 /**
  * NMSecretAgentDeleteSecretsFunc:
  * @agent: the secret agent object
- * @connection: the connection for which secrets were to be deleted
- * @error: if the deleting secrets failed, give a descriptive error here 
+ * @connection: (transfer none): the connection for which secrets were to be deleted,
+ * note that this object will be unrefed after the callback has returned, use
+ * g_object_ref()/g_object_unref() if you want to use this object after the callback
+ * has returned
+ * @error: if the deleting secrets failed, give a descriptive error here
  * @user_data: caller-specific data to be passed to the function
  *
  * Called as a result of a request by NM to delete secrets.  When the
@@ -186,12 +216,12 @@ typedef struct {
 
 	/* Called when the subclass should retrieve and return secrets.  Subclass
 	 * must copy or reference any arguments it may require after returning from
-	 * this method, as the arguments will freed (except for 'agent', 'callback',
+	 * this method, as the arguments will freed (except for 'self', 'callback',
 	 * and 'user_data' of course).  If the request is canceled, the callback
 	 * should still be called, but with the NM_SECRET_AGENT_ERROR_AGENT_CANCELED
 	 * error.
 	 */
-	void (*get_secrets) (NMSecretAgent *agent,
+	void (*get_secrets) (NMSecretAgent *self,
 	                     NMConnection *connection,
 	                     const char *connection_path,
 	                     const char *setting_name,
@@ -206,17 +236,17 @@ typedef struct {
 	 * call, sending the NM_SECRET_AGENT_ERROR/NM_SECRET_AGENT_ERROR_AGENT_CANCELED
 	 * error to that callback.
 	 */
-	void (*cancel_get_secrets) (NMSecretAgent *agent,
+	void (*cancel_get_secrets) (NMSecretAgent *self,
 	                            const char *connection_path,
 	                            const char *setting_name);
 
 	/* Called when the subclass should save the secrets contained in the
 	 * connection to backing storage.  Subclass must copy or reference any
 	 * arguments it may require after returning from this method, as the
-	 * arguments will freed (except for 'agent', 'callback', and 'user_data'
+	 * arguments will freed (except for 'self', 'callback', and 'user_data'
 	 * of course).
 	 */
-	void (*save_secrets) (NMSecretAgent *agent,
+	void (*save_secrets) (NMSecretAgent *self,
 	                      NMConnection *connection,
 	                      const char *connection_path,
 	                      NMSecretAgentSaveSecretsFunc callback,
@@ -225,10 +255,10 @@ typedef struct {
 	/* Called when the subclass should delete the secrets contained in the
 	 * connection from backing storage.  Subclass must copy or reference any
 	 * arguments it may require after returning from this method, as the
-	 * arguments will freed (except for 'agent', 'callback', and 'user_data'
+	 * arguments will freed (except for 'self', 'callback', and 'user_data'
 	 * of course).
 	 */
-	void (*delete_secrets) (NMSecretAgent *agent,
+	void (*delete_secrets) (NMSecretAgent *self,
 	                        NMConnection *connection,
 	                        const char *connection_path,
 	                        NMSecretAgentDeleteSecretsFunc callback,
