@@ -43,8 +43,8 @@ typedef struct {
 
 GType nm_keyfile_connection_get_type (void);
 
-NMKeyfileConnection *nm_keyfile_connection_new (const char *filename,
-                                                NMConnection *source,
+NMKeyfileConnection *nm_keyfile_connection_new (NMConnection *source,
+                                                const char *filename,
                                                 GError **error);
 
 const char *nm_keyfile_connection_get_path (NMKeyfileConnection *self);
